import boto3
import logging

from botocore.exceptions import ClientError

try:
    from aws_ecs_service.package.response import Response
    from aws_ecs_service.package.fix_kwargs import fix_kwargs
except ImportError:
    # Lambda specific import.
    # noinspection PyUnresolvedReferences
    from response import Response
    # noinspection PyUnresolvedReferences
    from fix_kwargs import fix_kwargs


logger = logging.getLogger()
logger.setLevel(logging.INFO)


class Action:
    @staticmethod
    @fix_kwargs
    def create(**kwargs) -> Response:
        """
        Runs and maintains a desired number of tasks from a specified task definition.
        If the number of tasks running in a service drops below the desiredCount , Amazon ECS runs
        another copy of the task in the specified cluster. To update an existing service, see UpdateService.

        In addition to maintaining the desired count of tasks in your service, you can
        optionally run your service behind one or more load balancers. The load balancers
        distribute traffic across the tasks that are associated with the service.

        Read more:
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ecs.html#ECS.Client.create_service

        :param kwargs: Create action arguments.

        :return: Response dictionary.
        """
        logger.info(f'Calling boto3 ecs client "create_service" with parameters: {kwargs}.')
        response = boto3.client('ecs').create_service(**kwargs)

        return Response(
            cluster=kwargs.get('cluster'),
            service_name=kwargs.get('serviceName'),
            success=True,
            metadata=response
        )

    @staticmethod
    @fix_kwargs
    def update(**kwargs) -> Response:
        """
        Modifies the parameters of a service.

        For services using the rolling update (ECS ) deployment controller, the desired count,
        deployment configuration, network configuration, or task definition used can be updated.

        For services using the blue/green (CODE_DEPLOY ) deployment controller, only the desired count,
        deployment configuration, and health check grace period can be updated using this API.
        If the network configuration, platform version, or task definition need to be updated,
        a new AWS CodeDeploy deployment should be created.

        Read more:
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ecs.html#ECS.Client.update_service

        :param kwargs: Update action arguments.

        :return: Response dictionary.
        """
        try:
            logger.info(f'Calling boto3 ecs client "update_service" with parameters: {kwargs}.')
            response = boto3.client('ecs').update_service(**kwargs)
        except ClientError as ex:
            if ex.response['Error']['Code'] == 'InvalidParameterException':
                if 'codedeploy' in ex.response['Error']['Message'].lower():
                    # For services using the blue/green (CODE_DEPLOY ) deployment controller,
                    # only the desired count, deployment configuration, and health check grace period
                    # can be updated using this API. If the network configuration, platform version, or task definition
                    # need to be updated, a new AWS CodeDeploy deployment should be created.
                    kwargs = dict(
                        cluster=kwargs.get('cluster'),
                        service=kwargs.get('service'),
                        desiredCount=kwargs.get('desiredCount'),
                        deploymentConfiguration=kwargs.get('deploymentConfiguration'),
                        healthCheckGracePeriodSeconds=kwargs.get('healthCheckGracePeriodSeconds'),
                    )

                    logger.info(f'Calling boto3 ecs client "update_service" for CODEDEPLOY with parameters: {kwargs}.')
                    response = boto3.client('ecs').update_service(**kwargs)
                else:
                    raise
            elif ex.response['Error']['Code'] == 'ServiceNotActiveException':
                # We can not update ecs service if it is inactive.
                response = {'status': 'ServiceNotActiveException'}
            elif ex.response['Error']['Code'] == 'ServiceNotFoundException':
                # If for some reason service was not found - don't update and return.
                response = {'status': 'ServiceNotFoundException'}
            else:
                raise

        return Response(
            cluster=kwargs.get('cluster'),
            service_name=kwargs.get('service'),
            success=True,
            metadata=response
        )

    @staticmethod
    @fix_kwargs
    def delete(**kwargs) -> Response:
        """
        Deletes a specified service within a cluster. You can delete a service if you have no
        running tasks in it and the desired task count is zero. If the service is actively
        maintaining tasks, you cannot delete it, and you must update the service to a desired
        task count of zero. For more information, see UpdateService.

        Read more:
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ecs.html#ECS.Client.delete_service

        :param kwargs: Delete action arguments.

        :return: Response dictionary.
        """
        try:
            logger.info('Making ecs desired count 0...')
            boto3.client('ecs').update_service(
                cluster=kwargs.get('cluster'),
                service=kwargs.get('service'),
                desiredCount=0,
            )
        except ClientError as ex:
            logger.error(
                f'Failed to set desired count to 0. Reason: {repr(ex)}, {ex.response}. '
                f'Ignoring exception and trying to delete ecs service anyways.'
            )
        except Exception as ex:
            logger.error(f'Unknown error: {repr(ex)}.')

        logger.info('Deleting service...')
        logger.info(f'Calling boto3 ecs client "delete_service" with parameters: {kwargs}.')
        response = boto3.client('ecs').delete_service(**kwargs)

        return Response(
            cluster=kwargs.get('cluster'),
            service_name=kwargs.get('service'),
            success=True,
            metadata=response
        )



import copy
import json
import logging
import boto3

from typing import Dict, Any, Optional
from botocore.exceptions import ClientError

logger = logging.getLogger()
logger.setLevel(logging.INFO)


class Response:
    def __init__(self, cluster: str, service_name: str, success: bool, metadata: Dict[str, Any]) -> None:
        self.__cluster = cluster
        self.__service_name = service_name
        self.__success = success
        self.__metadata = metadata

    @property
    def cluster(self) -> str:
        return self.__cluster

    @property
    def service_name(self) -> str:
        return self.__service_name

    @property
    def metadata(self) -> Dict[str, Any]:
        return copy.deepcopy(self.__metadata)

    @property
    def success(self) -> bool:
        return self.__success

    @property
    def service_arn(self) -> Optional[str]:
        try:
            response = boto3.client('ecs').describe_services(
                cluster=self.__cluster,
                services=[self.__service_name],
            )
        except ClientError as ex:
            logger.error(f'Failed to describe ecs services. Reason: {repr(ex)}, {ex.response}.')
            return None

        try:
            return response['services'][0]['serviceArn']
        except (KeyError, IndexError) as ex:
            logger.error(f'Failed to parse ecs response. Reason: {repr(ex)}.')
            return None

    def to_dict(self) -> Dict[str, Any]:
        response = {
            'arn': self.service_arn,
            'name': self.service_name,
            'success': self.success,
            'meta': self.metadata
        }

        return response



import logging

from typing import Dict, Any
from aws_cdk.aws_cloudformation import CustomResource, CustomResourceProvider, ICustomResourceProvider
from aws_cdk.aws_iam import Role, CompositePrincipal, ServicePrincipal, PolicyDocument, PolicyStatement, Effect
from aws_cdk.aws_lambda import IFunction, Runtime, Code, Function
from aws_cdk.core import Stack, RemovalPolicy, Duration
from aws_ecs_service.package import package_root

logr = logging.getLogger(__name__)


class EcsService:
    """
    Class that manages ecs service resource.

    Read more about ecs service:
    https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs_services.html
    """

    def __init__(
            self,
            scope: Stack,
            id: str,
            on_create_action: Dict[str, Any],
            on_update_action: Dict[str, Any],
            on_delete_action: Dict[str, Any],
    ) -> None:
        """
        Constructor.

        :param scope: CloudFormation stack in which resources should be placed.
        :param id: Name (id) or prefix for resources.
        :param on_create_action: Create action arguments. Read more on:
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ecs.html#ECS.Client.create_service
        :param on_update_action: Update action arguments. Read more on:
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ecs.html#ECS.Client.update_service
        :param on_delete_action: Delete action arguments. Read more on:
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ecs.html#ECS.Client.delete_service
        """
        self.__role = Role(
            scope=scope,
            id=id + 'Role',
            role_name=id + 'Role',
            assumed_by=CompositePrincipal(
                ServicePrincipal("lambda.amazonaws.com"),
                ServicePrincipal("cloudformation.amazonaws.com")
            ),
            inline_policies={
                id + 'Policy': PolicyDocument(
                    statements=[
                        PolicyStatement(
                            actions=[
                                'ecs:createService',
                                'ecs:updateService',
                                'ecs:deleteService',
                                'ecs:describeServices',
                                'ecs:listServices',
                                'ecs:updateServicePrimaryTaskSet'
                            ],
                            effect=Effect.ALLOW,
                            resources=['*']
                        ),
                        PolicyStatement(
                            actions=[
                                "logs:CreateLogGroup",
                                "logs:CreateLogStream",
                                "logs:PutLogEvents"
                            ],
                            effect=Effect.ALLOW,
                            resources=['*']
                        ),
                    ]
                )
            },
            managed_policies=[]
        )

        self.__custom_backend = Function(
            scope=scope,
            id=id + 'Backend',
            code=Code.from_asset(
                path=package_root
            ),
            handler='index.handler',
            runtime=Runtime.PYTHON_3_6,
            description=f'A custom resource backend to manage ecs {id} service.',
            function_name=id + 'Backend',
            memory_size=128,
            role=self.__role,
            timeout=Duration.seconds(900),
        )

        # noinspection PyTypeChecker
        provider: ICustomResourceProvider = CustomResourceProvider.from_lambda(self.__custom_backend)

        self.__custom_resource = CustomResource(
            scope=scope,
            id=id + 'CustomResource',
            provider=provider,
            removal_policy=RemovalPolicy.DESTROY,
            properties={
                'onCreate': on_create_action,
                'onUpdate': on_update_action,
                'onDelete': on_delete_action
            },
            resource_type='Custom::EcsService'
        )

        # Make sure that custom resource is deleted before lambda function backend.
        self.__custom_resource.node.add_dependency(self.__custom_backend)

    @property
    def arn(self) -> str:
        return self.__custom_resource.get_att('arn')

    @property
    def name(self) -> str:
        return self.__custom_resource.get_att('name')

    @property
    def backend(self) -> IFunction:
        return self.__custom_backend

    @property
    def custom_resource(self) -> CustomResource:
        return self.__custom_resource



from setuptools import setup, find_packages

with open('README.md') as readme_file:
    README = readme_file.read()

with open('HISTORY.md') as history_file:
    HISTORY = history_file.read()

setup(
    name='aws_ecs_service',
    version='1.2.0',
    license='GNU GENERAL PUBLIC LICENSE Version 3',
    packages=find_packages(exclude=['venv', 'test']),
    description=(
        'AWS CDK package that manages ecs service resource.'
    ),
    long_description=README + '\n\n' + HISTORY,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=[
        'aws_cdk.core>=1.60.0,<2.0.0',
        'aws_cdk.custom_resources>=1.60.0,<2.0.0',
        'aws_cdk.aws_lambda>=1.60.0,<2.0.0',
        'aws_cdk.aws_iam>=1.60.0,<2.0.0',
    ],
    author='Laimonas Sutkus',
    author_email='laimonas@idenfy.com, laimonas.sutkus@gmail.com',
    keywords='AWS CDK ECS CustomResource',
    url='https://github.com/idenfy/AwsEcsService.git',
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Operating System :: OS Independent',
    ],
)



import logging
import json

from botocore.vendored import requests

SUCCESS = "SUCCESS"
FAILED = "FAILED"
MAXIMUM_RESPONSE_SIZE = 1024 * 4


def send(event, context, responseStatus, responseData, physicalResourceId=None, noEcho=False, reason=None):
    responseUrl = event['ResponseURL']

    logging.info(f'Sending response back to CloudFormation: {responseUrl}...')

    responseBody = {}

    responseBody['Status'] = responseStatus
    responseBody['Reason'] = reason or ('See the details in CloudWatch Log Stream: ' + context.log_stream_name)
    responseBody['PhysicalResourceId'] = physicalResourceId or context.log_stream_name
    responseBody['StackId'] = event['StackId']
    responseBody['RequestId'] = event['RequestId']
    responseBody['LogicalResourceId'] = event['LogicalResourceId']
    responseBody['NoEcho'] = noEcho
    responseBody['Data'] = responseData

    response_json = json.dumps(responseBody, default=lambda o: '<not serializable>')
    is_too_big = len(response_json.encode('utf-8')) >= MAXIMUM_RESPONSE_SIZE

    if is_too_big:
        responseBody['Data'] = None
        response_json = json.dumps(responseBody, default=lambda o: '<not serializable>')

    headers = {
        'content-type': '',
        'content-length': str(len(response_json))
    }

    try:
        response = requests.put(
            responseUrl,
            data=response_json,
            headers=headers
        )

        logging.info("Status code: " + response.reason)
    except Exception as e:
        logging.info("send(..) failed executing requests.put(..): " + str(e))



import copy
import logging
import json

from typing import Dict, Any, Union, List, Tuple

logger = logging.getLogger()
logger.setLevel(logging.INFO)


def fix_kwargs(func):
    def wrapper_fix_kwargs(*args, **kwargs):
        args = list(copy.deepcopy(args))
        kwargs = dict(copy.deepcopy(kwargs))

        before_json = json.dumps(kwargs, default=lambda o: '<not serializable>')
        logger.info(f'Fixing kwargs... Before: {before_json}.')

        __fix(args)
        __fix(kwargs)

        after_json = json.dumps(kwargs, default=lambda o: '<not serializable>')
        logger.info(f'Fixing kwargs... After: {after_json}.')

        return func(*args, **kwargs)
    return wrapper_fix_kwargs


def __fix(data: Union[Tuple[Any, ...], List[Any], Dict[Any, Any]]) -> None:
    """
    Converts strings to ints (if possible).

    :param data: Data to convert.

    :return: No return.
    """
    if isinstance(data, dict):
        for key in data.keys():
            data[key] = try_to_int(data[key])
            data[key] = try_to_bool(data[key])
            __fix(data[key])
    if isinstance(data, list):
        for index in range(len(data)):
            data[index] = try_to_int(data[index])
            data[index] = try_to_bool(data[index])
            __fix(data[index])


def try_to_int(value: Any) -> Union[Any, int]:
    try:
        return int(value)
    except (ValueError, TypeError):
        return value


def try_to_bool(value: Any) -> Union[Any, int]:
    truthy = ['true', 'True', 'TRUE']
    falsy = ['false', 'False', 'FALSE']

    if value in truthy:
        return True

    if value in falsy:
        return False

    return value
